package com.qmth.ops.biz.ai.client;

import com.qmth.boot.core.ai.model.ocr.ParseDocTask;
import com.qmth.boot.core.ai.model.ocr.ParseDocTaskResult;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.rateLimit.service.RateLimiter;
import com.qmth.boot.core.rateLimit.service.impl.MemoryRateLimiter;
import okhttp3.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.Duration;

/**
 * 文档解析类接口基础实现
 */
public abstract class DocApiClient {

    private static final Logger log = LoggerFactory.getLogger(DocApiClient.class);

    private final OcrApiConfig config;

    private final OkHttpClient client;

    private RateLimiter queryRateLimiter;

    public DocApiClient(OcrApiConfig config) {
        this.config = config;

        OkHttpClient.Builder builder = new OkHttpClient.Builder().connectionPool(new ConnectionPool())
                .connectTimeout(Duration.ofSeconds(10)).readTimeout(Duration.ofSeconds(50));
        Interceptor interceptor = getInterceptor();
        if (interceptor != null) {
            builder.addInterceptor(interceptor);
        }
        this.client = builder.build();

        if (config.getQps() > 0) {
            this.queryRateLimiter = new MemoryRateLimiter(config.getQps(), 1000);
        }
    }

    public abstract ParseDocTask parseDocTask(byte[] fileData, String fileName) throws Exception;

    public abstract ParseDocTaskResult parseDocTaskQuery(String taskId) throws Exception;

    protected OcrApiConfig getConfig() {
        return config;
    }

    protected OkHttpClient getClient() {
        return client;
    }

    protected RateLimiter getQueryRateLimiter() {
        return queryRateLimiter;
    }

    protected Interceptor getInterceptor() {
        return null;
    }

    protected byte[] download(String url) {
        Request request = new Request.Builder().url(url).get().build();
        try (Response response = this.getClient().newCall(request).execute();) {
            ResponseBody respBody = response.body();
            if (response.isSuccessful() && respBody != null) {
                return respBody.bytes();
            }
            log.error("获取文件内容失败！responseCode:{}", response.code());
            throw new StatusException("获取文件内容失败！");
        } catch (Exception e) {
            throw new StatusException("获取文件内容失败！", e);
        }
    }

}
